# encoding: utf-8

'''
@author: vmmhypervisor
'''


import golib.common as common
import golib.sqlite as sql


class GitTag:
    TABLE_NAME = "tag"


class Git:

    class Database:
        NAME = "git commit"
        VERSION = "1.2.0"

    class TableName:
        COMMIT_DATA = "commit_data"
        COMMIT_ID = "commit_id"
        GIT_SHOW_BODY = "git_show_body"
        GIT_SHOW_NUMSTAT = "git_show_numstat"
        TAG = "tag"
        DIR_STRUCTURE = "dir_structure"
        MODIFICATION = "lines_of_modification"

    class Field:
        COMMIT_NO = "commit_no"
        BRANCH = "branch"
        COMMIT_HASH = "commit_hash"
        AUTHOR = "author"
        COMMIT_DATETIME = common.Field.DATETIME
        COMMIT_TITILE_LINE = "commit_title_line"
        FULL_COMMIT_MESSAGE = "full_commit_message"

        SOURCE = "source"
        COMMENT = "comment"
        CODE_TAG = "code_tag"
        COMMENT_TAG = "comment_tag"

        SOURCE_GROUP_LEVEL1 = "source group 1"
        SOURCE_GROUP_LEVEL2 = "source group 2"
        FILE_PATH = "file_path"
        FILE_ORDER = "file_order"
        DIR_DEPTH = "depth"
        PATH_ID = "path_id"
        IS_DIR = "is_dir"
        INSERTIONS = "insertions"
        DELETIONS = "deletions"
        SUBTOTAL = "subtotal"

        TAG_CATEGORY = "tag_category"
        TAG_VALUE = "tag_value"
        TAG_COUNT = "tag_count"

        GIT_SHOW_LIST = [
            COMMIT_NO,
            BRANCH,
            COMMIT_HASH,
            AUTHOR,
            COMMIT_DATETIME,
            COMMIT_TITILE_LINE,
            FULL_COMMIT_MESSAGE,
            CODE_TAG,
            COMMENT_TAG,
        ]
        GIT_SHOW_ATTR_LIST = [
            COMMIT_NO,
            BRANCH,
            COMMIT_HASH,
            AUTHOR,
            COMMIT_DATETIME,
            COMMIT_TITILE_LINE,
            FULL_COMMIT_MESSAGE,
            CODE_TAG,
            COMMENT_TAG,
            SOURCE,
            COMMENT,
        ]

        GIT_NUMSTAT_LIST = [
            COMMIT_HASH,
            AUTHOR,
            SOURCE_GROUP_LEVEL1,
            SOURCE_GROUP_LEVEL2,
            FILE_PATH,
            DIR_DEPTH,
            INSERTIONS,
            DELETIONS,
        ]

        GIT_TAG_LIST = [
            COMMIT_HASH,
            TAG_CATEGORY,
            TAG_VALUE,
            TAG_COUNT,
        ]

    class Const:
        FILE = 0
        DIR = 1


def getCommitLinesOfModificationTable(con, commit_hash):
    """
    対象コミット中の各ファイルの修正行数を格納した辞書を返す
    """

    where_commit_hash = sql.SqlQuery.make_where(
        Git.Field.COMMIT_HASH, commit_hash)
    dict_file_loc = {}

    # ディレクトリ単位の修正行数取得 --
    result = con.execute_select(
        select=sql.SqlQuery.to_attr_str(Git.Field.FILE_PATH),
        table=Git.TableName.DIR_STRUCTURE,
        where=sql.SqlQuery.make_where(Git.Field.IS_DIR, Git.Const.DIR))
    for dir_path in sql.getListFromQueryResult(result.fetchall()):
        where_list = [where_commit_hash]
        dir_name_list = dir_path.split("/")

        if len(dir_name_list) > 0:
            where_list.append(sql.SqlQuery.make_where(
                Git.Field.SOURCE_GROUP_LEVEL1, dir_name_list[0]))

        if len(dir_name_list) > 1:
            where_list.append(sql.SqlQuery.make_where(
                Git.Field.SOURCE_GROUP_LEVEL2, dir_name_list[1]))

        """
        select_query = "SUM(%s), SUM(%s)" % (
            sql.SqlQuery.to_attr_str(Git.Field.INSERTIONS),
            sql.SqlQuery.to_attr_str(Git.Field.DELETIONS),
        )
        #"""
        select_query = "SUM(%s)" % (
            sql.SqlQuery.to_attr_str(Git.Field.INSERTIONS))
        insertion_sum = con.getValue(
            select=select_query,
            table=Git.TableName.GIT_SHOW_NUMSTAT,
            where=" AND ".join(where_list))

        dict_file_loc[dir_path] = insertion_sum

    # 個々のファイルの修正行数取得 ---
    result = con.execute_select(
        select=",".join(sql.SqlQuery.to_attr_str_list([
            Git.Field.FILE_PATH, Git.Field.INSERTIONS])),
        table=Git.TableName.GIT_SHOW_NUMSTAT,
        where=where_commit_hash)
    total_insertion = 0
    for file_path, insertion in result.fetchall():
        dict_file_loc[file_path] = insertion
        if common.isInteger(insertion):
            total_insertion += insertion

    dict_file_loc[Git.Field.SUBTOTAL] = total_insertion

    return dict_file_loc
