'''
@author: vmmhypervisor
'''

import subprocess

import golib.common as common
from golib.logger import logger


SSH_BIN_PATH = "ssh"
RSH_BIN_PATH = "rsh"


class NetworkError(Exception):
    pass


def getNtpdateOffset(sys_wrapper, ntp_server):
    import re

    common.checkInstallCommand(["ntpdate"])

    proc = sys_wrapper.popenCommand("ntpdate -q " + ntp_server)
    if proc is None:
        return 0

    ret_stdout, ret_stderr = proc.communicate()
    if proc.returncode != 0:
        raise RuntimeError("rc=%d, msg=%s" % (proc.returncode, ret_stderr))

    match = re.search("offset [\d]+", ret_stdout, re.MULTILINE)
    if match is None:
        raise ValueError("offset time not found: " + ret_stdout)

    match = re.search("[\d]+", match.group(), re.MULTILINE)

    return int(match.group())


def makeRemoteCommand(remote_host, command, connect_bin=SSH_BIN_PATH):
    if common.isEmptyString(remote_host):
        logger.warn("null remote host")
        return ""

    if common.isEmptyString(command):
        logger.warn("null command")
        return ""

    if connect_bin == SSH_BIN_PATH:
        return "%s %s '%s'" % (connect_bin, remote_host, command)
    elif connect_bin == RSH_BIN_PATH:
        return "%s %s %s" % (connect_bin, remote_host, command)
    else:
        raise ValueError("unknown bin: " + str(connect_bin))


def pingTest(destination):
    repeat_count = 2
    return_code = 1

    for _i in range(repeat_count):
        command = "ping -c 1 %s" % (destination)
        process = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
        process.communicate()
        return_code = process.returncode
        if return_code == 0:
            return 0

    return return_code


def testPingMulti(destination_list):
    repeat_count = 2
    task_list = []

    for destination in destination_list:
        command = "ping -c %d %s" % (repeat_count, destination)
        logger.debug(command)
        process = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
        task_list.append([destination, process])

    dict_Dest_Result = {}
    for task in task_list:
        destination, process = task
        process.communicate()
        return_code = process.returncode
        dict_Dest_Result[destination] = return_code

    logger.debug("ping result: %s" % str(dict_Dest_Result))

    return dict_Dest_Result


def testSsh(destination, timeout=3):
    import os
    import signal
    import time

    command = "%s %s hostname > /dev/null 2>&1" % (SSH_BIN_PATH, destination)
    logger.debug(command)
    process = subprocess.Popen(
        command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    time.sleep(timeout)

    return_code = process.poll()
    if return_code is None:
        return_code = 1
        os.kill(process.pid, signal.SIGTERM)  # for compatibility
        os.kill(process.pid, 9)  # for compatibility

    return return_code


def testSshMulti(destination_list):
    import os
    import signal
    import time

    timeout = 3
    task_list = []

    for destination in destination_list:
        command = "%s %s hostname > /dev/null 2>&1" % (
            SSH_BIN_PATH, destination)
        logger.debug(command)
        process = subprocess.Popen(command, shell=True)
        task_list.append([destination, process])
    dict_Dest_Result = {}
    time.sleep(timeout)
    for task in task_list:
        destination, process = task
        return_code = process.poll()

        if return_code is None:
            return_code = 1
            os.kill(process.pid, signal.SIGTERM)  # for compatibility

            # process.terminate()

        dict_Dest_Result[destination] = return_code

    return dict_Dest_Result


def syncTimeWithNtp(sys_wrapper, ip_addr, ntp_server_ip_addr):
    logger.info("sychronize time: sync-server=%s, ntp-server=%s" % (
        ip_addr, ntp_server_ip_addr))

    ntp_service = "/etc/init.d/ntp"

    command = makeRemoteCommand(ip_addr, ntp_service + " stop")
    sys_wrapper.run(command)

    command = makeRemoteCommand(
        ip_addr, "ntpdate -b %s" % (ntp_server_ip_addr))
    sys_wrapper.run(command)

    command = makeRemoteCommand(ip_addr, ntp_service + " start")
    sys_wrapper.run(command)
