# encoding: utf-8

'''
@author: vmmhypervisor
'''

#import external.requests as requests
import requests

from golib.logger import logger


class Redmine:
    __SEPARATOR_FORMAT = "--- %s ------------------------------"

    @property
    def page_title(self):
        return self.__page_title

    @page_title.setter
    def page_title(self, value):
        self.__page_title = value

    def __init__(self, url, api_key, project_id, page_title):
        self.__base_url = url
        self.__api_key = api_key
        self.__project_id = project_id
        #self.__page_title	= page_title
        self.page_title = page_title

    def __getPageUrl(self):
        item_list = [
            self.__base_url,
            "projects", self.__project_id,
            "wiki", self.page_title,
            # wiki", self.__page_title,
        ]
        return "/".join(item_list) + ".json"

    def getWikiPageText(self):
        import json

        logger.debug(self.__SEPARATOR_FORMAT % "GET")

        dict_params = {
            "key"	: self.__api_key,
        }

        response = requests.get(self.__getPageUrl(), params=dict_params)
        logger.debug("GET URL: " + response.url)
        json_data = json.loads(response.text)
        text = json_data.get("wiki_page").get("text")

        logger.debug(self.__SEPARATOR_FORMAT % "GET end")

        return text

    def putWikiPage(self, text):
        import json

        logger.debug(self.__SEPARATOR_FORMAT % "PUT")
        logger.debug("PUT URL: " + self.__getPageUrl())

        dict_wiki_data = {
            "text": text,
        }
        dict_json = {
            "wiki_page": dict_wiki_data,
        }
        dict_header = {
            'Content-Type': 'application/json',
            'X-Redmine-API-Key': self.__api_key,
        }

        response = requests.put(
            self.__getPageUrl(),
            data=json.dumps(dict_json), headers=dict_header)
        for key, value in response.headers.items():
            logger.debug("%s | %s" % (key, value))
        logger.debug("response: " + response.text)

        logger.debug(self.__SEPARATOR_FORMAT % "PUT end")
