# encoding: utf-8

'''
@author: vmmhypervisor

:required:
    https://pypi.python.org/pypi/pytest
'''

import platform

import pytest

from golib.common import *
from golib.gfile import *
from golib.sqlite import *


nan = float("nan")
inf = float("inf")


def testStr():
    assert str(0) == "0"
    assert str(0.0) == "0.0"
    assert str(0.00) == "0.0"
    assert str(0.1) == "0.1"
    assert str(0.10) == "0.1"


def return0_func():
    return 0


class Test_isNotNone:

    @pytest.mark.parametrize(["value", "expected"], [
        ["nan", True],
        ["", True],
        ["テスト", True],
        [[], True],
        [1, True],
        [True, True],

        [None, False],
    ])
    def test_normal(self, value, expected):
        assert isNotNone(value) == expected


class Test_isString:

    @pytest.mark.parametrize(["value", "expected"], [
        ["", True],
        ["a1!" * 1000, True],
        ["テスト", True],

        [None, False],
        [nan, False],
        [1, False],
        [True, False],
    ])
    def test_normal(self, value, expected):
        assert isString(value) == expected


class Test_isInteger:

    @pytest.mark.parametrize(["value"], [
        [0], [99999999999], [-99999999999],
        [1234567890123456789L], [-1234567890123456789L],
        ["0"], ["99999999999"], ["-99999999999"],
        [" 1"], ["1 "],
    ])
    def test_normal(self, value):
        assert isInteger(value)

    @pytest.mark.parametrize(["value"], [
        [None], [nan], [inf],
        [0.5], ["0.5"],
        [""], ["test"], ["1a1"], ["11a"], ["a11"],
        ["１"],
        [True],
    ])
    def test_abnormal(self, value):
        assert not isInteger(value)


class Test_isHex:

    @pytest.mark.parametrize(["value"], [
        ["0x00"], ["0xffffffff"], ["a"], ["f"],
    ])
    def test_normal(self, value):
        assert isHex(value)

    @pytest.mark.parametrize(["value"], [
        [None], [nan], [inf],
        [0], [1], [0.5],
        ["test"], ["g"], ["１"],
        [True],
    ])
    def test_abnormal(self, value):
        assert not isHex(value)


class Test_isFloat:

    @pytest.mark.parametrize(["value"], [
        [0.0], [0.1], [-0.1], [1], [-1],
        ["0.0"], ["0.1"], ["-0.1"], ["1"], ["-1"],
        [.5], [0.],
        [nan], [inf],
    ])
    def test_normal(self, value):
        assert isFloat(value)

    @pytest.mark.parametrize(["value"], [
        [None],
        ["test"],
        ["inf"],
        ["１"],
        [True],
    ])
    def test_abnormal(self, value):
        assert not isFloat(value)


class Test_isNaN:

    @pytest.mark.parametrize(["value", "expected"], [
        [nan, True],

        [None, False],
        ["nan", False],
        ["１", False],
        [inf, False],
        [1, False],
        [0.1, False],
        [True, False],
    ])
    def test_normal(self, value, expected):
        assert isNaN(value) == expected


class Test_isNotEmptyString:

    @pytest.mark.parametrize(["value", "expected"], [
        ["nan", True],
        ["テスト", True],

        [None, False],
        ["", False],
        ["  ", False],
        ["\t", False],
        ["\n", False],
        [[], False],
        [1, False],
        [True, False],
    ])
    def test_normal(self, value, expected):
        assert isNotEmptyString(value) == expected


class Test_isEmptyString:

    @pytest.mark.parametrize(["value", "expected"], [
        [None, True],
        ["", True],
        ["  ", True],
        ["\t", True],
        ["\n", True],

        ["nan", False],
        ["テスト", False],
        [[], False],
        [1, False],
        [True, False],
    ])
    def test_normal(self, value, expected):
        assert isEmptyString(value) == expected


class Test_isList:

    @pytest.mark.parametrize(["value"], [
        [[]],
        [[1]],
        [["a", "b"]],
        [["a"] * 200000],
    ])
    def test_normal(self, value):
        assert isList(value)

    @pytest.mark.parametrize(["value"], [
        [None], [()],
        [nan], [0], ["aaa"],
        [True],
    ])
    def test_abnormal(self, value):
        assert not isList(value)


class Test_isTuple:

    @pytest.mark.parametrize(["value"], [
        [()],
        [(1,)],
        [("a", "b")],
        [("a",) * 200000],
    ])
    def test_normal(self, value):
        assert isTuple(value)

    @pytest.mark.parametrize(["value"], [
        [None], [[]],
        [nan], [0], ["aaa"],
        [True],
    ])
    def test_abnormal(self, value):
        assert not isTuple(value)


class Test_isListOrTuple:

    @pytest.mark.parametrize(["value", "expected"], [
        [[], True],
        [[1], True],
        [["a"] * 200000, True],
        [(), True],
        [(1,), True],
        [("a",) * 200000, True],

        [None, False],
        [nan, False],
        [0, False],
        ["aaa", False],
        [True, False],
    ])
    def test_normal(self, value, expected):
        assert isListOrTuple(value) == expected


class Test_isEmptyList:

    @pytest.mark.parametrize(["value", "expected"], [
        [[], True],
        [None, True],

        [[1], False],
        [["a"] * 200000, False],
    ])
    def test_normal(self, value, expected):
        assert isEmptyList(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        [(), False],
        [(1, 2), False],
        [nan, False],
        [0, False],
        ["aaa", False],
        [True, False],
    ])
    def test_abnormal(self, value, expected):
        assert isEmptyList(value) == expected


class Test_isEmptyTuple:

    @pytest.mark.parametrize(["value", "expected"], [
        [(), True],
        [None, True],

        [(1,), False],
        [("a",) * 200000, False],
    ])
    def test_normal(self, value, expected):
        assert isEmptyTuple(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        [[], False],
        [[1, 2], False],
        [nan, False],
        [0, False],
        ["aaa", False],
        [True, False],
    ])
    def test_abnormal(self, value, expected):
        assert isEmptyTuple(value) == expected


class Test_isEmptyListOrTuple:

    @pytest.mark.parametrize(["value", "expected"], [
        [(), True],
        [[], True],
        [None, True],

        [[1], False],
        [["a"] * 200000, False],
        [(1,), False],
        [("a",) * 200000, False],
    ])
    def test_normal(self, value, expected):
        assert isEmptyListOrTuple(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        [nan, False],
        [0, False],
        ["aaa", False],
        [True, False],
    ])
    def test_abnormal(self, value, expected):
        assert isEmptyListOrTuple(value) == expected


class Test_isNotEmptyListOrTuple:

    @pytest.mark.parametrize(["value", "expected"], [
        [(), False],
        [[], False],
        [None, False],

        [[1], True],
        [["a"] * 200000, True],
        [(1,), True],
        [("a",) * 200000, True],
    ])
    def test_normal(self, value, expected):
        assert isNotEmptyListOrTuple(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        [nan, False],
        [0, False],
        ["aaa", False],
        [True, False],
    ])
    def test_abnormal(self, value, expected):
        assert isNotEmptyListOrTuple(value) == expected


class Test_isDict:

    @pytest.mark.parametrize(["value"], [
        [{}],
        [{"a": 1, "b": 2}],
    ])
    def test_normal(self, value):
        assert isDict(value)

    @pytest.mark.parametrize(["value"], [
        [None],
        [(1, 2)],
        [[1, 2]],
        [0], ["aaa"],
        [True],
    ])
    def test_abnormal(self, value):
        assert not isDict(value)


class Test_isFunction:

    @pytest.mark.parametrize(["value"], [
        [return0_func],
    ])
    def test_normal(self, value):
        assert isFunction(value)

    @pytest.mark.parametrize(["value"], [
        [None],
        [0],
        ["aaa"],
        [True],
    ])
    def test_abnormal(self, value):
        assert not isFunction(value)


class Test_isInstallCommand:

    @pytest.mark.parametrize(["value", "expected"], [
        ["ls", True],
        ["", False],
        ["テスト", False],
        ["__not_existing_command__", False],
    ])
    def test_linux_normal(self, value, expected):
        assert isInstallCommand(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        ["dir", False],
    ])
    def test_windows(self, monkeypatch, value, expected):
        def mockreturn():
            return "Windows"

        monkeypatch.setattr(platform, 'system', mockreturn)

        assert isInstallCommand(value) == expected


class Test_verifyVarType:

    @pytest.mark.parametrize(["verify_func", "value"], [
        [isInteger, 0],
    ])
    def test_normal(self, verify_func, value):
        verifyVarType(verify_func, value)

    @pytest.mark.parametrize(["verify_func", "value", "expected"], [
        [isInteger, None, TypeError],
        [isInteger, 0.0, TypeError],
    ])
    def test_exception(self, verify_func, value, expected):
        with pytest.raises(expected):
            verifyVarType(verify_func, value)

    @pytest.mark.parametrize(["value", "verify_func", "expected"], [
        [None, isInteger, TypeError],
    ])
    def test_abnormal(self, value, verify_func, expected):
        with pytest.raises(expected):
            verifyVarType(value)


class Test_verifyFloat:

    @pytest.mark.parametrize(["value"], [
        [0], [-0],
        [123L], [-123L],
        [9.99], [-9.99],
        ["0"], ["-0"],
        ["9.99"], ["-9.99"],
        [99999999999999099999.99], [-99999999999999099999.99],
        ["99999999999999099999.99"], ["-99999999999999099999.99"],
    ])
    def test_normal(self, value):
        assert verifyFloat(value)

    @pytest.mark.parametrize(["value", 'exception'], [
        [None, TypeError],
        ["test", TypeError],
        ["a", TypeError],
        ["１", TypeError],
        ["0xff", TypeError],
        [nan, ValueError],
        [True, TypeError],
    ])
    def test_exception(self, value, exception):
        with pytest.raises(exception):
            verifyFloat(value)


class Test_safeDivision:

    @pytest.mark.parametrize(['dividend', "divisor", "expected"], [
        [4, 2, 2],
        [-4, -2, 2],
        [4, -2, -2],
        [-4, 2, -2],
        [2, 4, 0.5],
        [200000000000000, 400000000000000, 0.5],
        [0, 2, 0],
        ["1", 2, 0.5],
        [1, "2", 0.5],
    ])
    def test_normal(self, dividend, divisor, expected):
        assert safeDivision(dividend, divisor) == expected

    @pytest.mark.parametrize(['dividend', "divisor"], [
        [2, 0],
        [None, 1],
        [1, None],
        [None, None],
        ["a", 2],
        [1, "a"],
        ["a", "a"],
        [nan, 2],
        [2, nan],
        [nan, nan],
        [nan, inf],
        [inf, nan],
        [inf, inf],
        [True, True],
    ])
    def test_abnormal_1(self, dividend, divisor):
        assert isNaN(safeDivision(dividend, divisor))

    @pytest.mark.parametrize(['dividend', "divisor", "expected"], [
        [inf, 2, inf],
        [2, inf, 0],
    ])
    def test_abnormal_2(self, dividend, divisor, expected):
        assert safeDivision(dividend, divisor) == expected


class Test_getListItem:
    input_list = [1, 2, 3]

    @pytest.mark.parametrize(['input_list', "index", "expected"], [
        [input_list, 0, 1],
        [input_list, 2, 3],

        [input_list, -1, None],
        [input_list, 4, None],

        [input_list, 1.0, None],
        [input_list, True, None],
        [input_list, None, None],
        [input_list, nan, None],
        [input_list, inf, None],
    ])
    def test(self, input_list, index, expected):
        assert getListItem(input_list, index) == expected


class Test_getSignificantDigit:

    @pytest.mark.parametrize(["value", "expected"], [
        [0, 1], [-0, 1],
        [1.01, 1], [-1.01, 1],
        [9.99, 1], [-9.99, 1],
        ["9.99", 1], ["-9.99", 1],
        ["0", 1], ["-0", 1],

        [10, 2], [-10, 2],
        [99.99, 2], [-99.99, 2],
        ["10", 2], ["-10", 2],
        ["99.99", 2], ["-99.99", 2],

        [100, 3], [-100, 3],
        [999.99, 3], [-999.99, 3],
        ["100", 3], ["-100", 3],
        ["999.99", 3], ["-999.99", 3],

        [10000000000000000000, 20], [-10000000000000000000, 20],
        [99999999999999099999.99, 20], [-99999999999999099999.99, 20],
        ["10000000000000000000", 20], ["-10000000000000000000", 20],
        ["99999999999999099999.99", 20], ["-99999999999999099999.99", 20],
    ])
    def test_normal(self, value, expected):
        assert getSignificantDigit(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        [99999999999999999999.99, 21],
        [-99999999999999999999.99, 21],
        ["99999999999999999999.99", 21],
        ["-99999999999999999999.99", 21],
    ])
    def test_abnormal(self, value, expected):
        # expected result == 20
        assert getSignificantDigit(value) == expected

    @pytest.mark.parametrize(["value", 'exception'], [
        [None, TypeError],
        [True, TypeError],
        ["test", TypeError],
        ["a", TypeError],
        ["0xff", TypeError],
        [nan, ValueError],
        [inf, ValueError],
    ])
    def test_exception(self, value, exception):
        with pytest.raises(exception):
            getSignificantDigit(value)


class Test_getFloatDigit:

    @pytest.mark.parametrize(["value", "expected"], [
        [0, 0], [-0, 0],
        [1000, 0], [-1000, 0],
        [1000.0, 0], [-1000.0, 0],

        [0.0, 1], [-0.0, 1],
        [0.10, 1], [-0.10, 1],
        [0.9, 1], [-0.9, 1],
        [10.1, 1], [-10.1, 1],
        [100.1, 1], [-100.1, 1],

        [0.01, 2], [-0.01, 2],
        [0.99, 2], [-0.99, 2],
        [99.999, 2], [-99.999, 2],

        [0.001, 3], [-0.001, 3],
        [9.999, 3], [-9.999, 3],
        [9.9999, 3], [-9.9999, 3],

        [0.0123, 4], [-0.0123, 4],
        [0.01234, 4], [-0.01234, 4],
    ])
    def test_normal(self, value, expected):
        assert getFloatDigit(value, getSignificantDigit(value)) == expected

    @pytest.mark.parametrize(["value", 'significant_digit', "expected"], [
        [None, 1, TypeError],
        [False, 1, TypeError],
        [nan, 1, ValueError],
        [inf, 1, ValueError],

        [1.0, False, TypeError],
        [1.0, None, TypeError],
        [None, None, TypeError],
        [nan, None, TypeError],
        [inf, None, TypeError],
    ])
    def test_abnormal(self, value, significant_digit, expected):
        with pytest.raises(expected):
            getFloatDigit(value, significant_digit)


class Test_getNumberOfDigit:

    @pytest.mark.parametrize(["value", "expected"], [
        [0, (1, 0)], [-0, (1, 0)], ["0", (1, 0)], ["-0", (1, 0)],
        [10, (2, 0)], [-10, (2, 0)], ["10", (2, 0)], ["-10", (2, 0)],
        [10.1, (2, 1)], [-10.1, (2, 1)], ["10.1",
                                          (2, 1)], ["-10.1", (2, 1)],
        [0.1, (1, 1)], [-0.1, (1, 1)], ["0.1", (1, 1)], ["-0.1", (1, 1)],
        [0.01, (1, 2)], [-0.01, (1, 2)], ["0.01",
                                          (1, 2)], ["-0.01", (1, 2)],
        [0.001, (1, 3)], [-0.001, (1, 3)], ["0.001",
                                            (1, 3)], ["-0.001", (1, 3)],
    ])
    def test_normal(self, value, expected):
        assert getNumberOfDigit(value) == expected

    @pytest.mark.parametrize(["value"], [
        [None], [nan], [True],
        ["0xff"], ["test"], ["テスト"],
    ])
    def test_abnormal(self, value):
        sig_digit, float_digit = getNumberOfDigit(value)
        assert isNaN(sig_digit)
        assert isNaN(float_digit)


class Test_getTextLen:

    def test_normal_1(self):
        assert getTextLen("") == 0
        assert getTextLen(
            "aaaaaaaaaaaaaaaaaaaa"
            "aaaaaaaaaaaaaaaaaaaa"
            "aaaaaaaaaaaaaaaaaaaa"
            "aaaaaaaaaaaaaaaaaaaa"
            "aaaaaaaaaaaaaaaaaaaa"
        ) == 100

    def test_normal_2(self):
        assert getTextLen(u"あ") == 1

    def test_abnormal_1(self):
        assert getTextLen(None) == 4

    def test_abnormal_2(self):
        assert getTextLen(nan) == 3
        assert getTextLen(inf) == 3


class Test_convertValue:

    def test_normal_1(self):
        assert convertValue("0") == 0
        assert convertValue("9999999999") == 9999999999
        assert convertValue("-9999999999") == -9999999999
        assert convertValue(0) == 0
        assert convertValue(9999999999) == 9999999999
        assert convertValue(-9999999999) == -9999999999

    def test_normal_2(self):
        assert convertValue("0.0") == 0
        assert convertValue(0.0) == 0

    def test_normal_3(self):
        assert convertValue("aaaaa") == "aaaaa"

    def test_abnormal_1(self):
        assert convertValue(None) is None

    def test_abnormal_2(self):
        assert isNaN(convertValue(nan))
        assert convertValue(inf) == inf


class Test_removeItemFromList:

    def test_normal_1(self):
        arg_list = [1, 2, 3]

        assert removeItemFromList(arg_list, 2)
        assert arg_list == [1, 3]

    def test_abnormal_1(self):
        arg_list = [1, 2, 3]

        assert not removeItemFromList(arg_list, 4)
        assert arg_list == [1, 2, 3]

    def test_abnormal_2(self):
        with pytest.raises(TypeError):
            assert not removeItemFromList(None, 4)


class Test_removeListFromList:

    def test_normal_1(self):
        arg_list = [1, 2, 3, 4]
        remove_list = [2, 3]

        removeListFromList(arg_list, remove_list)
        assert arg_list == [1, 4]

    def test_abnormal_1(self):
        arg_list = [1, 2, 3]
        remove_list = [4, 5]

        removeListFromList(arg_list, remove_list)
        assert arg_list == [1, 2, 3]


class Test_toByte:

    @pytest.mark.parametrize(["value", "expected"], [
        ["b", 1], ["B", 1],
        ["k", 1024 ** 1], ["K", 1024 ** 1],
        ["m", 1024 ** 2], ["M", 1024 ** 2],
        ["g", 1024 ** 3], ["G", 1024 ** 3],
        ["t", 1024 ** 4], ["T", 1024 ** 4],
        ["p", 1024 ** 5], ["P", 1024 ** 5],
    ])
    def test_normal(self, value, expected):
        assert toByte(value) == expected

    @pytest.mark.parametrize(["value", 'exception'], [
        [None, TypeError],
        [True, TypeError],
        [nan, TypeError],
        ["a", ValueError],
        ["ｂ", ValueError],
    ])
    def test_exception(self, value, exception):
        with pytest.raises(exception):
            toByte(value)


class Test_convertHumanReadableToByte:

    def test_normal_1(self):
        assert convertHumanReadableToByte("2b") == 2
        assert convertHumanReadableToByte("2k") == 2 * 1024
        assert convertHumanReadableToByte("2m") == 2 * 1024 ** 2
        assert convertHumanReadableToByte("2g") == 2 * 1024 ** 3

    def test_abnormal_1(self):
        arg_list = [
            None,
        ]

        for arg in arg_list:
            with pytest.raises(TypeError):
                assert toByte(arg) == 1

    def test_abnormal_2(self):
        arg_list = [
            "a",
            "10 k", "10kb",
        ]

        for arg in arg_list:
            with pytest.raises(ValueError):
                assert toByte(arg) == 1


class Test_convertBytesToHumanReadable:

    def test_normal_1(self):
        assert convertBytesToHumanReadable(2) == "2 B"
        assert convertBytesToHumanReadable(2 * 1024 ** 1) == "2 KB"
        assert convertBytesToHumanReadable(2 * 1024 ** 2) == "2 MB"
        assert convertBytesToHumanReadable(2 * 1024 ** 3) == "2 GB"
        assert convertBytesToHumanReadable(2 * 1024 ** 4) == "2 TB"

    def test_normal_2(self):
        assert convertBytesToHumanReadable(int(2.5 * 1024 ** 2)) == "2.5 MB"

    def test_abnormal_1(self):
        arg_list = [
            1.0,
            None,
            nan,
            inf,
        ]

        for arg in arg_list:
            with pytest.raises(TypeError):
                convertBytesToHumanReadable(arg)


class Test_strtobool_wrapper:

    @pytest.mark.parametrize(["value", "expected"], [
        ["true", True],
        ["True", True],
        ["TRUE", True],
        ["false", False],
        ["False", False],
        ["FALSE", False],
    ])
    def test_normal(self, value, expected):
        assert strtobool_wrapper(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        ["falsea", ValueError],
        ["aFalse", ValueError],
        ["aFALSEa", ValueError],
        ["Ｔｒｕｅ", ValueError],

        [1.0, TypeError],
        [None, TypeError],
        [nan, TypeError],
    ])
    def test_exception(self, value, expected):
        with pytest.raises(expected):
            strtobool_wrapper(value)


class Test_dict_to_matrix:

    @pytest.mark.parametrize(["value", "expected"], [
        [{"a": 1, "b": 2}, [["a", 1], ["b", 2]]],
        [{}, []]
    ])
    def test_normal(self, value, expected):
        assert dict_to_matrix(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        [1.0, TypeError],
        [None, TypeError],
        [nan, TypeError],
    ])
    def test_exception(self, value, expected):
        with pytest.raises(expected):
            dict_to_matrix(value)


class Test_splitLineList:

    @pytest.mark.parametrize(["value", "separator", "expected"], [
        [
            ["a", "", "b", "c"],
            "",
            [["a"], ["b", "c"]]
        ],
        [
            ("a", "", "b", "c"),
            "",
            [["a"], ["b", "c"]]
        ],
        [
            ["a", "", "b", "c"],
            "b",
            [["a", ""], ["c"]]
        ],
    ])
    def test_normal(self, value, separator, expected):
        assert splitLineList(value, separator) == expected

    @pytest.mark.parametrize(["value", "separator", "expected"], [
        [None, "", TypeError],
        ["", "", TypeError],
        [1, "", TypeError],
    ])
    def test_exception(self, value, separator, expected):
        with pytest.raises(expected):
            splitLineList(value, separator)


class Test_splitLineListByRe:

    @pytest.mark.parametrize(
        ["value", "separator", "is_include_matched_line", "expected"],
        [
            [
                ["abcdefg", "ABCDEFG", "1234"],
                re.compile("DEFG$"),
                False,
                [["abcdefg"], ["1234"]],
            ],
            [
                ["abcdefg", "ABCDEFG", "1234"],
                re.compile("DEFG$"),
                True,
                [["abcdefg"], ["ABCDEFG", "1234"]],
            ],
        ])
    def test_normal(self, value, separator, is_include_matched_line, expected):
        assert splitLineListByRe(
            value, separator, is_include_matched_line) == expected

    @pytest.mark.parametrize(["value", "separator", "expected"], [
        [None, "", TypeError],
        ["", "", TypeError],
        [1, "", TypeError],
    ])
    def test_exception(self, value, separator, expected):
        with pytest.raises(expected):
            splitLineList(value, separator)


class Test_checkInstallCommand:

    @pytest.mark.parametrize(["value"], [
        [[]],
    ])
    def test_normal(self, value):
        checkInstallCommand(value)

    @pytest.mark.parametrize(["value", "expected"], [
        [None, TypeError],
        ["__not_existing_command__", NotInstallError],
    ])
    def test_exception(self, value, expected):
        with pytest.raises(expected):
            checkInstallCommand(value)


class Test_getFileNameFromCommand:

    @pytest.mark.parametrize(["value", "expected"], [
        ["/bin/ls", "bin-ls"],
        ["cat /proc/cpuinfo", "cat_-proc-cpuinfo"],
    ])
    def test_normal(self, value, expected):
        assert getFileNameFromCommand(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        [None, TypeError],
        ["", TypeError],
    ])
    def test_exception(self, value, expected):
        with pytest.raises(expected):
            getFileNameFromCommand(value)


class Test_compareVersion:

    @pytest.mark.parametrize(['lhs', 'rhs', "expected"], [
        ["1.0.0", "1.0.0", 0],

        ["1.0.0", "0.9.9", 1],
        ["1.1.0", "1.0.9", 1],
        ["1.0.1", "1.0.0", 1],

        ["0.9.9", "1.0.0", -1],
        ["1.0.9", "1.1.0", -1],
        ["1.0.0", "1.0.1", -1],
    ])
    def test_normal(self, lhs, rhs, expected):
        assert compareVersion(lhs, rhs) == expected

    @pytest.mark.parametrize(['lhs', 'rhs', "expected"], [
        [None, None, TypeError],
        ["1.0.0", None, TypeError],
        [None, "1.0.0", TypeError],
        [1, "1.0.0", TypeError],
        [nan, "1.0.0", TypeError],
        [True, "1.0.0", TypeError],
        ["aaa", "1.0.0", ValueError],
        ["1.2.3.4", "1.0.0", ValueError],
    ])
    def test_exception(self, lhs, rhs, expected):
        with pytest.raises(expected):
            compareVersion(lhs, rhs)


class Test_sleepWrapper:

    @pytest.mark.parametrize(["value", "dry_run"], [
        [0.1, False],
        [0.1, True],
    ])
    def test_normal(self, value, dry_run):
        sleepWrapper(value, dry_run)

    @pytest.mark.parametrize(["value", "dry_run"], [
        [-1, False],
    ])
    def test_abnormal(self, value, dry_run):
        sleepWrapper(value, dry_run)

    @pytest.mark.parametrize(["value", "dry_run", "expected"], [
        [None, False, TypeError],
        ["1.0", False, TypeError],
    ])
    def test_exception(self, value, dry_run, expected):
        with pytest.raises(expected):
            sleepWrapper(value, dry_run)


class Test_dumpDict:

    @pytest.mark.parametrize(["value", "expected"], [
        [
            {"a": 1},
            """{\n    \"a\": 1\n}""",
        ],
    ])
    def test_normal(self, value, expected):
        assert dumpDict(value) == expected

    @pytest.mark.parametrize(["value", "expected"], [
        ["", ""],
        [None, ""],
    ])
    def test_abnormal(self, value, expected):
        assert dumpDict(value) == expected
